#include "enemyship.h"

//------------------------------------------------------------------------------

//Constructor
CEnemyShip::CEnemyShip(int l, DATAFILE *d)
{
    Level = l;
    SpriteData = d;
    Remove = Exploding = false;
    switch (Level)
    {
        case 1:
            Sprite = (BITMAP*)SpriteData[ENEMY1].dat;
            dY = -1.3;
            MaxShields = 2;
            Acceleration = 0.04;
            MaxSpeed = 1.3;
            ReloadTime = 150;
            break;
        case 2:
            Sprite = (BITMAP*)SpriteData[ENEMY2].dat;
            MaxShields = 3;
            dY = -1.5;
            Acceleration = 0.05;
            MaxSpeed = 1.5;
            ReloadTime = 100;
            break;
        case 3:
            Sprite = (BITMAP*)SpriteData[ENEMY3].dat;
            MaxShields = 4;
            Acceleration = 0.06;
            MaxSpeed = 1.9;
            dY = -1.9;
            ReloadTime = 70;
            break;
    }
    Expl = new CExplosion(300, 0.6, X, Y, 0.02, (BITMAP*)SpriteData[SPARK].dat);
    Shields = MaxShields;
    Reset();
}
//------------------------------------------------------------------------------

//Destructor
CEnemyShip::~CEnemyShip()
{
    delete Expl;
}
//------------------------------------------------------------------------------

//Update
void CEnemyShip::Update()
{
    //Update explosion
    Expl->Update();
    if (Shields <= 0 && !Expl->Exploding())
        Remove = true;
    if (Expl->Exploding())
        return;
    
    //Move
    X += dX;
    Y += dY;
    if (X < DestX && dX < MaxSpeed)
        dX += Acceleration;
    else if (X > DestX && dX > -MaxSpeed)
        dX -= Acceleration;
    if (dX < 0.01 && dX > -0.01)
        dX = 0;
    if (abs(X-DestX) < 1)
        DestX = rand()%(SCREEN_W-30);
    if (Y < -Sprite->h)
        Reset();
    //Update bullets
    Reload--;
    if (Reload <= 0 && Y < SCREEN_H)
    {
        play_sample((SAMPLE*)Globals::SfxDat[LASERSAMPLE].dat, 150, 128, 1000, false);
        Weapon.push_back(new Bullet(X, Y));
        Reload = ReloadTime;
    }
    for (Bit i = Weapon.begin(); i != Weapon.end(); ++i)
    {
        (*i)->Y-=5;
        if ((*i)->Y < -4)
            (*i)->Remove = true;
    }
    for (Bit i = Weapon.begin(); i != Weapon.end();)
    {
        if ((*i)->Remove)
            i = Weapon.erase(i);
        else
            ++i;
    }
}
//------------------------------------------------------------------------------

//Draw
void CEnemyShip::Draw(BITMAP *Dest)
{
    if (!Expl->Exploding()) 
    {
        draw_sprite(Dest, Sprite, (int)X, (int)Y);
        //Draw bullets
        for (Bit i = Weapon.begin(); i != Weapon.end(); ++i)
            draw_sprite(Dest, (BITMAP*)SpriteData[LASER].dat, (int)(*i)->X, (int)(*i)->Y);
    }
    else
        Expl->Draw(Dest);
}
//------------------------------------------------------------------------------

//Reset everything
void CEnemyShip::Reset()
{
    X = rand() % SCREEN_W;
    Y = SCREEN_H*1.5 + rand() % SCREEN_H;
    dX = 0;
    //Shields = MaxShields;
    DestX = rand()%SCREEN_W;
    Reload = rand() % ReloadTime;
}
//------------------------------------------------------------------------------

//Take damage, return the level if dead
int CEnemyShip::TakeDamage(int Dmg)
{   
    if (!Expl->Exploding())
    {
        play_sample((SAMPLE*)Globals::SfxDat[HITSAMPLE].dat, 200, 128, 1000, false);
        Shields -= Dmg;
        if (Shields <= 0)
        {
            Exploding = true;
            play_sample((SAMPLE*)Globals::SfxDat[EXPLSAMPLE].dat, 200, 128, 1000, false);
            Expl->Reset(MidX(), MidY());
            Expl->Start();
            return Level;
        }
    }
    return 0;
}
//------------------------------------------------------------------------------
