#include "explosion.h"

//------------------------------------------------------------------------------

//Constructor
CExplosion::CExplosion(int num, float force, float x, float y, float dl, 
                BITMAP *Sprite)
{
    NumParticles = num;
    Particles = new CParticle[NumParticles](Sprite);
    Force = force;
    X = x;
    Y = y;
    dLife = dl;
    Started = false;
    Reset(X, Y);
}
//------------------------------------------------------------------------------

//Destructor
CExplosion::~CExplosion()
{
    delete []Particles;
}
//------------------------------------------------------------------------------

//Start the explosion
void CExplosion::Start()
{
    Started = true;
}
//------------------------------------------------------------------------------


//Stop the explosion
void CExplosion::Stop()
{
    Started = false;
}
//------------------------------------------------------------------------------

//Reset the explosion
void CExplosion::Reset(float x, float y)
{
    X = x;
    Y = y;

    for (int i = 0; i < NumParticles; ++i)
    {
        Particles[i].Reset(X, Y, 
                Force*sin((float)i+Rand(0.2,1))*Rand(0,1), 
                Force*cos((float)i+Rand(0.2,1))*Rand(0,1), 
                Rand(dLife*0.7, dLife*1.3));
    }
}
//------------------------------------------------------------------------------

//Update
void CExplosion::Update()
{
    if (Started)
    {
        Started = false;
        for (int i = 0; i < NumParticles; ++i)
        {
            Particles[i].Update();
            if (!Particles[i].IsDead())
                Started = true;
        }
    }
}
//------------------------------------------------------------------------------

//Draw
void CExplosion::Draw(BITMAP *Dest)
{
    if (Started)
    {
        for (int i = 0; i < NumParticles; ++i)
            Particles[i].Draw(Dest);
    }
}
//------------------------------------------------------------------------------

//Ranged random
float CExplosion::Rand(float Min, float Max)
{
    return Min + ((float)rand()/(float)RAND_MAX)*(Max-Min);
}
//------------------------------------------------------------------------------
