#include "gamemanager.h"

//------------------------------------------------------------------------------

//Constructor
CGameManager::CGameManager()
{
    //Load data
    Sprites = load_datafile("data/sprites.dat");
    if (!Sprites)
    {
        set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
        allegro_message("Unable to load file 'sprites.dat'");
        exit(1);
    } 
    AstSprites = load_datafile("data/astsprites.dat");
    if (!AstSprites)
    {
        set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
        allegro_message("Unable to load file 'astsprites.dat'");
        exit(1);
    } 
    GuiData = load_datafile("data/guidat.dat");
    if (!GuiData)
    {
        set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
        allegro_message("Unable to load file 'guidat.dat'");
        exit(1);
    } 
    Globals::SfxDat = load_datafile("data/sfx.dat");
    if (!Globals::SfxDat)
    {
        set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
        allegro_message("Unable to load file 'sfx.dat'");
        exit(1);
    } 

    //Create stuff
    StarField = new CStarField(50, Sprites);
    Menu = new CMenuManager(GuiData, (FONT*)Sprites[SMALLFONT].dat);
    Buffer = create_bitmap(SCREEN_W, SCREEN_H);
    Highscores = new CHighscoreManager(5, false);
    Highscores->ReadFromFile("highscores");
}
//------------------------------------------------------------------------------

//Destructor
CGameManager::~CGameManager()
{
    //Save highscores
    Highscores->PrintToFile("highscores");
    //Free stuff
    delete StarField;
    delete Menu;
    delete Highscores;
    destroy_bitmap(Buffer);
    unload_datafile(Sprites);
    unload_datafile(AstSprites);
    unload_datafile(GuiData);
    unload_datafile(Globals::SfxDat);
}
//------------------------------------------------------------------------------

//Run menus and game
void CGameManager::Run()
{
    int Choice, Score;
    
    while (true)
    {
        Choice = Menu->MainMenu();
        switch (Choice)
        {
            case 0:
                Menu->ViewStory();
                Score = GameLoop();
                if (Highscores->IsOnList(Score))
                {
                    Menu->AddHighscoreEntry(Highscores, Score);
                    Menu->ShowHighscores(Highscores);
                }
                break;
            case 1:
                Menu->ShowHighscores(Highscores);
                break;
            case 2:
                return;
        }
    }
}
//------------------------------------------------------------------------------

//Run the game loop
int CGameManager::GameLoop()
{
    //Reset everything
    while (key[KEY_ESC]);
    Globals::Counter = 0;
    Globals::Frames = Globals::FPS = 0;
    Player = new CPlayerShip(Sprites, &Asteroids, &Enemies);
    int Wave = 0, NewWaveCtr = 0;
    bool Paused = false, PauseKey = false;;
    BITMAP *NextWave = (BITMAP*)Sprites[NEXTWAVE].dat;
    //Add a few enemies
    for (int i = 0; i < 2; ++i) 
        Enemies.push_back(new CEnemyShip(1, (DATAFILE*)Sprites));
    
    while (!key[KEY_ESC] && !Player->Dead())
    {
        //Update logic
        while (Globals::Counter > 0)
        {
            //New wave
            if (!Paused)
            {
                if (Enemies.empty())
                {
                 NewWave(Wave);   
                    NewWaveCtr = 255;
                }
                //Update asteroids and enemies
                for (Ait i = Asteroids.begin(); i != Asteroids.end(); ++i)
                    (*i)->Update();
                for (Eit i = Enemies.begin(); i != Enemies.end(); ++i)
                    (*i)->Update();
                for (Eit i = Enemies.begin(); i != Enemies.end();)
                {
                    if ((*i)->Remove)
                    {
                        delete *i;
                        i = Enemies.erase(i);
                    }
                    else
                        ++i;
                }
                Player->Update();
                StarField->Update();
                if (NewWaveCtr > 0)
                    NewWaveCtr-=2;
            }
            //Check if paused
            if (key[KEY_P] && !PauseKey)
                Paused = !Paused;
            PauseKey = key[KEY_P];
            
            Globals::Counter--;
        }
        //Update graphics
        clear(Buffer);
        StarField->Draw(Buffer);
        Player->Draw(Buffer);
        for (Ait i = Asteroids.begin(); i != Asteroids.end(); ++i)
            (*i)->Draw(Buffer);
        for (Eit i = Enemies.begin(); i != Enemies.end(); ++i)
            (*i)->Draw(Buffer);
        if (NewWaveCtr > 0)
        {
            set_add_blender(0, 0, 0, NewWaveCtr);
            draw_trans_sprite(Buffer, NextWave, SCREEN_W/2-NextWave->w/2, SCREEN_H/2);
        }
        if (Paused)
            textprintf_centre(Buffer, (FONT*)GuiData[MENUFONT].dat, SCREEN_W/2,
                SCREEN_H/2, makecol(255, 255, 255), "GAME PAUSED - PRESS P TO RESUME");
        Player->DrawPanel(Buffer);
        draw_sprite(screen, Buffer, 0, 0);
        Globals::Frames++;
    }
    //Free stuff
    int Score = Player->GetScore();
    delete Player;
    for (Ait i = Asteroids.begin(); i != Asteroids.end(); ++i)
        delete *i;
    for (Eit i = Enemies.begin(); i != Enemies.end(); ++i)
        delete *i;
    Asteroids.clear();
    Enemies.clear();
    return Score;
}
//------------------------------------------------------------------------------

//Create a new level
void CGameManager::NewWave(int &W)
{
    W++;
    Asteroids.push_back(new CAsteroid(AstSprites, (BITMAP*)Sprites[SPARK].dat));
    int EnemyLvl = W/4; 
    if (EnemyLvl > 3)
        EnemyLvl = 3;
    for (int i = 0; i < W+1; ++i)
        Enemies.push_back(new CEnemyShip(1+EnemyLvl, (DATAFILE*)Sprites));
}
//------------------------------------------------------------------------------
