#include "highscore.h"

//------------------------------------------------------------------------------

//Constructor
CHighscoreManager::CHighscoreManager(int _EntryCount, bool _Reversed)
{
    //Number of entries
    EntryCount = _EntryCount;
    //If reversed is true, small number will be highest
    Reversed = _Reversed;
    //Fill with empty entries
    for (int i = 0; i < EntryCount; ++i)
    {
        HighscoreEntry TempEntry;
        TempEntry.Name = "None";
        if (Reversed)
            TempEntry.Score = 100;
        else
            TempEntry.Score = 0;
        TempEntry.Reversed = Reversed;
        Entries.push_back(TempEntry);
    }
}
//------------------------------------------------------------------------------


//Destructor
CHighscoreManager::~CHighscoreManager()
{
}
//------------------------------------------------------------------------------

//Add a new entry
void CHighscoreManager::AddEntry(string Name, int Score)
{
    if (IsOnList(Score))
    {
        HighscoreEntry TempEntry;
        TempEntry.Name = Name;
        TempEntry.Score = Score;
        TempEntry.Reversed = Reversed;
        Entries.pop_back();
        Entries.push_back(TempEntry);
        Entries.sort();
    }
}
//------------------------------------------------------------------------------

//Save to file
bool CHighscoreManager::PrintToFile(string Filename)
{
    //Write a temporary file
    ofstream Fout("Tempout.txt");
    if (!Fout)
        return false;
    list<HighscoreEntry>::iterator it = Entries.begin();    
    
    while (it != Entries.end())
    {
        Fout << it->Name << " " << it->Score << "\n";
        ++it;
    }
    Fout.close();
    
    //Crypt
    Fout.open((char*)Filename.c_str(), ios::out |ios::binary);
    ifstream Fin("Tempout.txt", ios::in | ios::binary);
    char Cchar;
    while(Fin.get(Cchar))
        Fout.put((char)~Cchar);
    Fout.close();
    Fin.close();
        
    remove("Tempout.txt");
    
    return true;
}
//------------------------------------------------------------------------------

//Load from file
bool CHighscoreManager::ReadFromFile(string Filename)
{
    //Create a temporary decrypted file
    ifstream Fin((char*)Filename.c_str(), ios::in | ios::binary);
    ofstream Fout("Tempin.txt", ios::out | ios::binary);
    char Cchar;
    while(Fin.get(Cchar))
        Fout.put((char)~Cchar);
    Fin.close();
    Fout.close();
    
    //Read the temporary file
    ifstream Fin2("Tempin.txt");
    if (!Fin2)
        return false;
    list<HighscoreEntry>::iterator it = Entries.begin();  
    string TempName;
    int TempScore; 
    
    while (it != Entries.end())
    {
        Fin2 >> it->Name >> it->Score;
        ++it;
    }
    Fin2.close();
    
    remove("Tempin.txt");
    
    return true;
}
//------------------------------------------------------------------------------

//Check if a score qualifies
bool CHighscoreManager::IsOnList(int Score)
{
    if (Reversed)
        return Score < Entries.back().Score;
    else
        return Score > Entries.back().Score;
}
//------------------------------------------------------------------------------

//Number of names on the list
unsigned int CHighscoreManager::Size()
{
    int Count = 0;
    list<HighscoreEntry>::iterator it;
    for (it = Entries.begin(); it != Entries.end(); ++it)
        Count++;
    return Count;
}
//------------------------------------------------------------------------------

//Get a specific entry
HighscoreEntry CHighscoreManager::GetEntry(int Index)
{
    list<HighscoreEntry>::iterator it = Entries.begin();
    int i = 0;
    while (i < Index)
    {
        ++i;
        ++it;
    }
    return *it;
}
//------------------------------------------------------------------------------

//Compare operator for entries
bool operator < (const HighscoreEntry &n1, const HighscoreEntry &n2)
{
    if (n1.Reversed)
        return n1.Score < n2.Score;
    else
        return n1.Score > n2.Score;
}
//------------------------------------------------------------------------------
