#include "menu.h"

//------------------------------------------------------------------------------

//Constructor
CMenuManager::CMenuManager(DATAFILE *d, FONT *f)
{
    Data = d;
    SmallFont = f;
    Cursor = (BITMAP*)Data[CURSOR].dat;
    //Create buttons
    BITMAP *Up = (BITMAP*)Data[BUTTONUP].dat, *Down = (BITMAP*)Data[BUTTONDOWN].dat;
    PlayBtn = new CGuiButton(SCREEN_W/2-Up->w/2, SCREEN_H/2, Up, Down, "Play",
                (FONT*)Data[MENUFONT].dat, makecol(0, 0, 0));
    HighscoreBtn = new CGuiButton(SCREEN_W/2-Up->w/2, SCREEN_H/2+Up->h, Up, 
            Down, "Hall of Fame", (FONT*)Data[MENUFONT].dat, makecol(0, 0, 0));
    ExitBtn = new CGuiButton(SCREEN_W/2-Up->w/2, SCREEN_H/2+2*Up->h, Up, Down, 
            "Exit", (FONT*)Data[MENUFONT].dat, makecol(0, 0, 0));
    int TextL = text_length((FONT*)Data[MENUFONT].dat, "AAAAAAAAAAAAAAA");
    HighscoreStrn = new CGuiString(SCREEN_W/2-TextL/2, 300, 15, "Anonymous", 
        (FONT*)Data[MENUFONT].dat, makecol(255, 255, 255), makecol(100, 100, 100));
    //Setup starfield
    NumStars = 80;
    Stars1 = new CStar[NumStars];
    Stars2 = new CStar[NumStars];
    Stars3 = new CStar[NumStars];
    for (int i = 0; i < NumStars; ++i)
    {
        Stars1[i].X = rand() % SCREEN_W;
        Stars1[i].Y = rand() % SCREEN_H;
        int C = 150 + rand() % 155;
        Stars1[i].Col = makecol(C, C, C);

        Stars2[i].X = rand() % SCREEN_W;
        Stars2[i].Y = rand() % SCREEN_H;
        C = 100 + rand() % 100;
        Stars2[i].Col = makecol(C, C, C);
        
        Stars3[i].X = rand() % SCREEN_W;
        Stars3[i].Y = rand() % SCREEN_H;
        C = 50 + rand() % 150;
        Stars3[i].Col = makecol(C, C, C);
    }
}
//------------------------------------------------------------------------------

//Destructor
CMenuManager::~CMenuManager()
{
    delete PlayBtn;
    delete HighscoreBtn;
    delete ExitBtn;
    delete HighscoreStrn;
    delete []Stars1;
    delete []Stars2;
    delete []Stars3;
}
//------------------------------------------------------------------------------

//Run the main menu
int CMenuManager::MainMenu()
{
    while (keypressed())readkey();
    Globals::Counter = 0; //Reset timer
    Buffer = create_bitmap(SCREEN_W, SCREEN_H);
    int Choice = -1;
    ExitBtn->Text = "Exit";
    //Logo
    BITMAP *Logo1 = (BITMAP*)Data[LOGO1].dat, *Logo2 = (BITMAP*)Data[LOGO2].dat;
    int Logo1X = SCREEN_W+10, Logo2X = Logo1X+Logo1->w+300;
    
    while (!key[KEY_ESC] && Choice < 0)
    {
        while (Globals::Counter > 0)
        {
            //Move the logo
            if (Logo1X > 40)
                Logo1X-=10;
            if (Logo2X > Logo1X+Logo1->w-10)
                Logo2X-=10;
            PlayBtn->Update();
            HighscoreBtn->Update();
            ExitBtn->Update();
            //Check buttons
            if (PlayBtn->ClickFlag)
            {
                PlayBtn->ClickFlag = false;
                Choice = 0;
            }
            if (HighscoreBtn->ClickFlag)
            {
                HighscoreBtn->ClickFlag = false;
                Choice = 1;
            }
            if (ExitBtn->ClickFlag)
            {
                ExitBtn->ClickFlag = false;
                Choice = 2;
            }
            MoveStars();
            Globals::Counter--;
        }
        clear(Buffer);
        DrawStars();
        textprintf(Buffer, SmallFont, 10, SCREEN_H-20, makecol(100, 100, 100), 
                        "2003 TINS entry by Hannes Jensen");
        PlayBtn->Draw(Buffer);
        HighscoreBtn->Draw(Buffer);
        ExitBtn->Draw(Buffer);
        draw_sprite(Buffer, Logo1, Logo1X, 10);
        draw_sprite(Buffer, Logo2, Logo2X, 10);
        draw_sprite(Buffer, Cursor, mouse_x, mouse_y);
        draw_sprite(screen, Buffer, 0, 0);
    }
    destroy_bitmap(Buffer);
    return Choice;
}
//------------------------------------------------------------------------------

//Show highscores
void CMenuManager::ShowHighscores(CHighscoreManager *Highscores)
{
    while (keypressed())readkey();
    Globals::Counter = 0; //Reset timer
    Buffer = create_bitmap(SCREEN_W, SCREEN_H);
    ExitBtn->Text = "Back";
    FONT *MenuFont = (FONT*)Data[MENUFONT].dat;
    
    while (!ExitBtn->ClickFlag)
    {
        while (Globals::Counter > 0)
        {
            ExitBtn->Update();
            MoveStars();
            Globals::Counter--;
        }
        clear(Buffer);
        DrawStars();
        ExitBtn->Draw(Buffer);
        textprintf_centre(Buffer, MenuFont, SCREEN_W/2, 170, makecol(255, 255, 255), "HALL OF FAME");
        for (int i = 0; i < Highscores->Size(); ++i)
        {
            textprintf_centre(Buffer, MenuFont, SCREEN_W/2, 200+i*20, 
                makecol(255-i*40, 255-i*40, 255-i*40), "%s, %d", 
                Highscores->GetEntry(i).Name.c_str(), Highscores->GetEntry(i).Score);
        }
        draw_sprite(Buffer, Cursor, mouse_x, mouse_y);
        draw_sprite(screen, Buffer, 0, 0);
    }
    
    ExitBtn->ClickFlag = false;
    destroy_bitmap(Buffer);  
}
//------------------------------------------------------------------------------

//Read a nnew name for the highscore list
void CMenuManager::AddHighscoreEntry(CHighscoreManager *Highscores, int Score)
{
    while (keypressed())readkey();
    Globals::Counter = 0;
    Buffer = create_bitmap(SCREEN_W, SCREEN_H);
    ExitBtn->Text = "OK";
    FONT *MenuFont = (FONT*)Data[MENUFONT].dat;
    
    while (!ExitBtn->ClickFlag)
    {
        while (Globals::Counter > 0)
        {
            HighscoreStrn->Update();
            ExitBtn->Update();
            MoveStars();
            Globals::Counter--;
        }
        clear(Buffer);
        DrawStars();
        ExitBtn->Draw(Buffer);
        HighscoreStrn->Draw(Buffer);
        textprintf_centre(Buffer, MenuFont, SCREEN_W/2, 240, makecol(255, 255, 255),
            "ENTER YOUR NAME:");
        draw_sprite(Buffer, Cursor, mouse_x, mouse_y);
        draw_sprite(screen, Buffer, 0, 0);
    }
    
    Highscores->AddEntry(HighscoreStrn->Text, Score);
    
    ExitBtn->ClickFlag = false;
    destroy_bitmap(Buffer);  
}
//------------------------------------------------------------------------------

//View the scrolling text in the beggining
void CMenuManager::ViewStory()
{
    while (keypressed())readkey();
    Globals::Counter = 0;
    Buffer = create_bitmap(SCREEN_W, SCREEN_H);
    FONT *MenuFont = (FONT*)Data[MENUFONT].dat;
    int NumStrings = 18;
    float TextY = SCREEN_H;
    string Strings[] = {
        "THE STORY SO FAR...",
        "(Press any key to skip)",
        "",
        "For as long as you can remember you",
        "lived peacefully in the northern parts",
        "of the planet Mars with your Martian wife",
        "and your Martian kids.",
        "But suddenly everything changed. The",
        "space filled with strange-looking aliens,",
        "invaders from Earth, coming to conquer",
        "your home planet. Seeing your chance to",
        "become a hero, you strap on your pilot",
        "goggles and enter your good old UFO,",
        "armed with carbon dioxide machine guns,",
        "ammonium nitrate rockets and highly",
        "explosive nitroglycerine jelly.",
        "Can you defend your red home from",
        "the invaders?"
    };
    
    while (!keypressed() && TextY > -NumStrings*20+20)
    {
        while (Globals::Counter > 0)
        {
            TextY -= 0.2;
            MoveStars();
            Globals::Counter--;
        }
        clear(Buffer);
        DrawStars();
        for (int i = 0; i < NumStrings; ++i)
            textprintf(Buffer, MenuFont, 170, (int)TextY+i*20, makecol(255, 255, 255),
                    Strings[i].c_str());
        draw_sprite(screen, Buffer, 0, 0);
    }

    destroy_bitmap(Buffer);  
}
//------------------------------------------------------------------------------

//Move the starfield
void CMenuManager::MoveStars()
{
    for (int i = 0; i < NumStars; ++i)
    {
        Stars1[i].X -= 0.7;
        if (Stars1[i].X < 0) Stars1[i].X = SCREEN_W;
        
        Stars2[i].X -= 0.4;
        if (Stars2[i].X < 0) Stars2[i].X = SCREEN_W;
        
        Stars3[i].X -= 0.1;
        if (Stars3[i].X < 0) Stars3[i].Y = SCREEN_W;
    } 
}
//------------------------------------------------------------------------------ 

//Draw the starfield
void CMenuManager::DrawStars()
{
    for (int i = 0; i < NumStars; ++i)
    {
        putpixel(Buffer, (int)Stars1[i].X, (int)Stars1[i].Y, Stars1[i].Col);
        putpixel(Buffer, (int)Stars2[i].X, (int)Stars2[i].Y, Stars2[i].Col);
        putpixel(Buffer, (int)Stars3[i].X, (int)Stars3[i].Y, Stars3[i].Col);
    }
}
//------------------------------------------------------------------------------ 
