#include "playership.h"

//------------------------------------------------------------------------------

//Constructor
CPlayerShip::CPlayerShip(DATAFILE *s,  vector<CAsteroid*> *a, vector<CEnemyShip*> *e)
{
    Astds = a;
    Enemies = e;
    SpriteData = s;
    Sprites = new BITMAP *[7];
    Shields = 100;
    Score = 0;
    //Assign bitmaps
    for (int i = 0; i < 7; ++i)
        Sprites[i] = (BITMAP*)SpriteData[UFO1+i].dat;
    //Coordinates and other stuff
    SpriteIndex = 3;
    X = SCREEN_W/2-Sprites[SpriteIndex]->w/2;
    Y = 30;
    dX = dY = 0.0;
    Acceleration = Globals::PlayerAcc;
    MaxSpeed = Globals::PlayerMaxSpeed;
    //Create stuff
    Fire = new CTrail(100, X, Y, 0.4, 0.05, (BITMAP*)SpriteData[SPARK].dat);
    Guns = new CWeaponManager(MachineGun, SpriteData, Astds, Enemies);
    Rockets = new CWeaponManager(Rocket, SpriteData, Astds, Enemies);
    Jelly = new CWeaponManager(JellyLauncher, SpriteData, Astds, Enemies);
    SelectedWeapon = Guns;
    Expl = new CExplosion(300, 0.6, X, Y, 0.02, (BITMAP*)SpriteData[SPARK].dat);
}
//------------------------------------------------------------------------------

//Destructor
CPlayerShip::~CPlayerShip()
{
    delete Guns;
    delete Rockets;
    delete Jelly;
    delete []Sprites;
    delete Fire;
    delete Expl;
}
//------------------------------------------------------------------------------

//Update the ship
void CPlayerShip::Update()
{
    //Read keys
    if (key[Globals::PlKeyUp] && dY > -MaxSpeed)
        dY -= Acceleration;
    if (key[Globals::PlKeyDown] && dY < MaxSpeed)
        dY += Acceleration;
    if (key[Globals::PlKeyRight] && dX < MaxSpeed)
        dX += Acceleration;
    if (key[Globals::PlKeyLeft] && dX > -MaxSpeed)
        dX -= Acceleration;  
    static bool Changing = false;
    if (key[Globals::PlKeyChange] && !Changing)
    {
        if (SelectedWeapon == Guns)
            SelectedWeapon = Rockets;
        else if (SelectedWeapon == Rockets)
            SelectedWeapon = Jelly;
        else
            SelectedWeapon = Guns;
    }Changing = key[Globals::PlKeyChange];
    //Slow down
    if (!key[Globals::PlKeyUp] && !key[Globals::PlKeyDown])
    {
        if (dY > 0)
            dY -= Acceleration;
        if (dY < 0)
            dY += Acceleration;
        if (dY > -2*Acceleration && dY < 2*Acceleration)
            dY = 0;
    }  
    if (!key[Globals::PlKeyLeft] && !key[Globals::PlKeyRight])
    {
        if (dX > 0)
            dX -= Acceleration;
        if (dX < 0)
            dX += Acceleration;
        if (dX > -2*Acceleration && dX < 2*Acceleration)
            dX = 0;
    }   
    //Determine what image to use
    SpriteIndex = (int)(7 * (dX+MaxSpeed)/(2*MaxSpeed));
    if (SpriteIndex > 6) SpriteIndex = 6;
    if (SpriteIndex < 0) SpriteIndex = 0;
    //Move
    X += dX;
    Y += dY;
    //Update weapons
    if (!Expl->Exploding())
    {
        if (key[Globals::PlKeyFire])
            SelectedWeapon->Fire(MidX(), MidY());
        Score += Guns->Update();
        Score += Rockets->Update();
        Score += Jelly->Update();
    }
    //Update trail
    Fire->Update(X+Sprites[SpriteIndex]->w/2-3, Y);
    if (dY > 0) Fire->Start();
    else Fire->Stop();
    Expl->Update();
    //Check collision
    CheckCollision();
}
//------------------------------------------------------------------------------

//Draw the ship
void CPlayerShip::Draw(BITMAP *Dest)
{ 
    if (Expl->Exploding())
        Expl->Draw(Dest);
    else
    {   
        Fire->Draw(Dest);
        Guns->Draw(Dest);
        Rockets->Draw(Dest);
        Jelly->Draw(Dest);
        draw_sprite(Dest, Sprites[SpriteIndex], (int)X, (int)Y);
    }
}
//------------------------------------------------------------------------------

//Draw the info panel
void CPlayerShip::DrawPanel(BITMAP *Dest)
{
    //Draw panel
    text_mode(-1);
    BITMAP *Panel = (BITMAP*)SpriteData[PANEL].dat;
    draw_sprite(Dest, Panel, 0, SCREEN_H-Panel->h);
     //Reload time
    textprintf(Dest, (FONT*)SpriteData[SMALLFONT].dat, 10, SCREEN_H-20, 
        makecol(0, 0, 0), "Reload: ");
    rectfill(Dest, 55, SCREEN_H-15, 107, SCREEN_H-11, makecol(0, 0, 0));
    rectfill(Dest, 56, SCREEN_H-14, 56+(int)(SelectedWeapon->Reloaded()*50.0), 
            SCREEN_H-12, makecol(150, 150, 150));
    //Weapon
    textprintf(Dest, (FONT*)SpriteData[SMALLFONT].dat, 10, SCREEN_H-40, 
        makecol(0, 0, 0), "Weapon: ");
    draw_sprite(Dest, SelectedWeapon->NameBmp(), 60, SCREEN_H-35); 
    //Shields
    textprintf(Dest, (FONT*)SpriteData[SMALLFONT].dat, 10, SCREEN_H-60, 
        makecol(0, 0, 0), "Shields: ");
    rectfill(Dest, 55, SCREEN_H-55, 107, SCREEN_H-51, makecol(0, 0, 0));
    rectfill(Dest, 56, SCREEN_H-54, 56+(int)(((float)Shields/100.0)*50.0), 
            SCREEN_H-52, makecol(0, 150, 0));
    //Score
    textprintf(Dest, (FONT*)SpriteData[SMALLFONT].dat, 10, SCREEN_H-80,
        makecol(100, 0, 0), "Score: %d", Score);
}
//------------------------------------------------------------------------------


//Check collision with asteroids
void CPlayerShip::CheckCollision()
{
    //Check edges
    if (X < 0)
    {
        X = 0;
        dX = 0;
    }
    if (X+Sprites[SpriteIndex]->w > SCREEN_W)
    {
        X = SCREEN_W-Sprites[SpriteIndex]->w;
        dX = 0;
    }
    if (Y < 0)
    {
        Y = 0;
        dY = 0;
    }
    if (Y+Sprites[SpriteIndex]->h > SCREEN_W)
    {
        Y = SCREEN_H-Sprites[SpriteIndex]->h;
        dY = 0;
    }

    //Check asteroids
    for (Ait i = Astds->begin(); i != Astds->end(); ++i)
    {
        float Dist = sqrt(pow(MidX()-(*i)->MidX(), 2)+pow(MidY()-(*i)->MidY(), 2));
        if (Dist < Radius()+(*i)->Radius())
        {
            TakeDamage(10+rand()%3);
            (*i)->Explode();
        }
    }
    
    //Check enemies
    for (Eit i = Enemies->begin(); i != Enemies->end(); ++i)
    {
        float Dist = sqrt(pow(MidX()-(*i)->MidX(), 2)+pow(MidY()-(*i)->MidY(), 2));
        if (Dist < Radius()+(*i)->Radius() && !(*i)->Exploding)
        {
            TakeDamage(10+rand()%3);
            (*i)->TakeDamage(100);
        }    
        //Check enemy bullets
        for (Bit b = (*i)->Weapon.begin(); b != (*i)->Weapon.end(); ++b)
        {
            if ((*b)->X > X && (*b)->X < X+Sprites[SpriteIndex]->w)
                if ((*b)->Y+4 > Y && (*b)->Y < Y+Sprites[SpriteIndex]->h)
                {
                    if (!(*i)->Exploding)
                    {
                        (*b)->Remove = true;
                        TakeDamage(5+rand()%3);
                    }
                }
        }
    }
}
//------------------------------------------------------------------------------

//Decrease shields and check if dead
void CPlayerShip::TakeDamage(int d)
{
    Shields -= d;
    play_sample((SAMPLE*)Globals::SfxDat[HITSAMPLE].dat, 200, 128, 1000, false);
    if (Shields <= 0)
    {
        if (!Expl->Exploding())
        {
            Expl->Reset(X+Sprites[SpriteIndex]->w/2, Y+Sprites[SpriteIndex]->h/2);
            Expl->Start();
            play_sample((SAMPLE*)Globals::SfxDat[EXPLSAMPLE].dat, 200, 128, 1000, false);
        }
        Shields = 0;
    }
}
//------------------------------------------------------------------------------
