#ifndef PLAYERSHIP_H
#define PLAYERSHIP_H

#include <cmath>
#include <vector>
#include <cstdlib>
#include "gameobject.h"
#include "data/sprites.h"
#include "trail.h"
#include "asteroid.h"
#include "enemyship.h"
#include "weaponmanager.h"

using namespace std;

//The player
class CPlayerShip : public TGameObject
{
    public:
        CPlayerShip(DATAFILE *s, vector<CAsteroid*> *a, vector<CEnemyShip*> *e);
        ~CPlayerShip();
        void Update();
        void Draw(BITMAP *Dest);
        void DrawPanel(BITMAP *Dest);
        void CheckCollision();
        void TakeDamage(int d);
        bool Dead(){return (Shields <= 0 && !Expl->Exploding());}
        int GetScore() {return Score;}
        
        //For collision detection
        float MidX() {return X+Sprites[SpriteIndex]->w/2;}
        float MidY() {return Y+Sprites[SpriteIndex]->h/2;}
        float Radius() {return Sprites[SpriteIndex]->h/2;}
    private:
        //Acceleration and velocity
        float Acceleration, MaxSpeed;
        int Shields, Score;
        //Sprites
        BITMAP **Sprites;
        int SpriteIndex;
        //Particle effects
        CExplosion *Expl;
        CTrail *Fire;
        //Weapons
        CWeaponManager *Guns;
        CWeaponManager *Rockets;
        CWeaponManager *Jelly;
        CWeaponManager *SelectedWeapon;
        //Data
        DATAFILE *SpriteData;
        //To do collision checking
        vector<CAsteroid*> *Astds;
        vector<CEnemyShip*> *Enemies;
};

typedef vector<CAsteroid*>::iterator Ait;
typedef vector<CEnemyShip*>::iterator Eit;
typedef vector<Bullet*>::iterator Bit;

#endif
