#include "weapon.h"

//------------------------------------------------------------------------------

//Constructor
CWeapon::CWeapon(WeaponType t, float x, float y, DATAFILE *d, vector<CAsteroid*> *a,
            vector<CEnemyShip*> *e)
{
    Type = t;
    X = x;
    Y = y;
    SpriteData = d;
    Astds = a;
    Enemies = e;
    dX = 0;
    Remove = false;
    Angle = 0;
    //Set values depending on type
    //It's all hard-coded, I don't have time to make anything flexible
    switch(Type)
    {
        case MachineGun:
            dY = 4;
            Sprite = (BITMAP*)SpriteData[BULLET].dat;
            Expl = new CExplosion(1, 0.2, X, Y, 0.05, (BITMAP*)SpriteData[SPARK].dat);
            Damage = 1;
            break;
        case Rocket:
            dY = 2.8;
            Sprite = (BITMAP*)SpriteData[ROCKET].dat;
            Fire = new CTrail(50, X, Y, 0.2, 0.04, (BITMAP*)SpriteData[SPARK].dat);
            Fire->Start();
            Damage = 2;
            Expl = new CExplosion(50, 0.5, X, Y, 0.03, (BITMAP*)SpriteData[SPARK].dat);
            break; 
        case JellyLauncher:
            dY = 2.4;
            Expl = new CExplosion(200, 0.6, X, Y, 0.02, (BITMAP*)SpriteData[SPARK].dat);
            Damage = 10;
            Sprite = (BITMAP*)SpriteData[JELLY].dat;
            break;   
    }
}
//------------------------------------------------------------------------------

//Destructor
CWeapon::~CWeapon()
{
    if (Type == Rocket)
        delete Fire;
}
//------------------------------------------------------------------------------

//Update return level of enemy killed
int CWeapon::Update()
{
    //Move
    X += dX;
    Y += dY;
    Angle++;
    //Update particle systems
    if (Type == Rocket)
        Fire->Update(X, Y-5);
    if (Expl->Exploding())
    {
        Expl->Update();
        if (!Expl->Exploding())
            Remove = true;
    }
    //Check collision
    if (!Expl->Exploding())
        return CheckCollision();
    return 0;
}
//------------------------------------------------------------------------------

//Draw
void CWeapon::Draw(BITMAP *Dest)
{
    if (Expl->Exploding())
        Expl->Draw(Dest);
    else
    {
        if (Type == Rocket)
            Fire->Draw(Dest);
        if (Type == JellyLauncher)
            rotate_sprite(Dest, Sprite, (int)X, (int)Y, itofix(Angle));
        else
            draw_sprite(Dest, Sprite, (int)X, (int)Y);
    }
}
//------------------------------------------------------------------------------

//Check collision
int CWeapon::CheckCollision()
{
    int Score = 0;

    //Check asteroids
    for (Ait i = Astds->begin(); i != Astds->end(); ++i)
    {
        if (X+Sprite->w > (*i)->MidX()-(*i)->Radius() && X < (*i)->MidX()+(*i)->Radius())
            if (Y+Sprite->h > (*i)->MidY()-(*i)->Radius() && Y < (*i)->MidY()+(*i)->Radius())
            {
                Expl->Reset(X+Sprite->w/2,  Y+Sprite->h/2);
                Expl->Start();   
                if (Type == Rocket || Type == JellyLauncher)
                    (*i)->Explode(); 
            }
    }
    //Check enemies
    for (Eit i = Enemies->begin(); i != Enemies->end(); ++i)
    {
        if (X+Sprite->w > (*i)->MidX()-(*i)->Radius() && X < (*i)->MidX()+(*i)->Radius())
            if (Y+Sprite->h > (*i)->MidY()-(*i)->Radius() && Y < (*i)->MidY()+(*i)->Radius())
            {
                Expl->Reset(X+Sprite->w/2,  Y+Sprite->h/2);
                Expl->Start();   
                Score += (*i)->TakeDamage(Damage); 
            }
    }
    //Check edges
    if (Y > SCREEN_H)
        Remove = true;
        
    return Score;
}
//------------------------------------------------------------------------------
