#include "weaponmanager.h"

//------------------------------------------------------------------------------

//Constructor
CWeaponManager::CWeaponManager(WeaponType t, DATAFILE *d, vector<CAsteroid*> *a,
                vector<CEnemyShip*> *e)
{
    Astds = a;
    Enemies = e;
    SpriteData = d;
    Type = t;
    switch (Type)
    {
        case MachineGun:
            ReloadTime = 15;
            break;
        case Rocket:
            ReloadTime = 60;
            break;
        case JellyLauncher:
            ReloadTime = 200;
            break;
    }
    Reload = 0;
}
//------------------------------------------------------------------------------

//Constructor - init for enemies
CWeaponManager::CWeaponManager(WeaponType t, DATAFILE *d)
{
    SpriteData = d;
    Type = t;
    switch (Type)
    {
        case MachineGun:
            ReloadTime = 15;
            break;
        case Rocket:
            ReloadTime = 70;
            break;
        case JellyLauncher:
            ReloadTime = 200;
            break;
    }
    Reload = 0;
}
//------------------------------------------------------------------------------

//Destructor
CWeaponManager::~CWeaponManager()
{
    for (Wit i = Bullets.begin(); i != Bullets.end(); ++i)
        delete *i;
}
//------------------------------------------------------------------------------

//Add a new bullet
void CWeaponManager::Fire(float x, float y)
{
    if (Reload <= 0)
    {
        Reload = ReloadTime;
        switch (Type)
        {
            case MachineGun:
                Bullets.push_back(new CWeapon(MachineGun, x, y, SpriteData, Astds, Enemies));
                play_sample((SAMPLE*)Globals::SfxDat[GUNSAMPLE].dat, 100, 128, 1000, false);
                break;
            case Rocket:
                Bullets.push_back(new CWeapon(Rocket, x, y, SpriteData, Astds, Enemies));
                play_sample((SAMPLE*)Globals::SfxDat[ROCKETSAMPLE].dat, 200, 128, 1000, false);
                break;
            case JellyLauncher:
                Bullets.push_back(new CWeapon(JellyLauncher, x, y, SpriteData, Astds, Enemies));
                play_sample((SAMPLE*)Globals::SfxDat[JELLYSAMPLE].dat, 200, 128, 1000, false);
                break;
        }
    }
}
//------------------------------------------------------------------------------

//Update all - return total score of enemies killed
int CWeaponManager::Update()
{
    int Score = 0;

    if (Reload > 0)
        Reload--;
    for (Wit i = Bullets.begin(); i != Bullets.end(); ++i)
        Score += (*i)->Update();
    for (Wit i = Bullets.begin(); i != Bullets.end();)
    {
        if ((*i)->Remove)
        {
            delete *i;
            i = Bullets.erase(i);
        }
        else
            ++i;
    }
    
    return Score;
}
//------------------------------------------------------------------------------

//Name of the weapon
BITMAP *CWeaponManager::NameBmp()
{
    switch (Type)
        {
            case MachineGun:
                return (BITMAP*)SpriteData[GUNTEXT].dat;
            case Rocket:
                return (BITMAP*)SpriteData[ROCKETTEXT].dat;
            case JellyLauncher:
                return (BITMAP*)SpriteData[JELLYTEXT].dat;
        }
}
//------------------------------------------------------------------------------

//Draw all
void CWeaponManager::Draw(BITMAP *Dest)
{
    for (Wit i = Bullets.begin(); i != Bullets.end(); ++i)
        (*i)->Draw(Dest);
}
//------------------------------------------------------------------------------
